#ifndef STL_FORWARD_LIST_WRAPPER_H
#define STL_FORWARD_LIST_WRAPPER_H

#include <forward_list>

namespace seq
{
   class STLForwardListWrapper: public Sequence
   {
   public:
      bool empty() const { return this->f.empty(); }
      size_t size() const {
         size_t count_elements = 0;
         for(auto j = this->f.begin(); j != this->f.end(); j++) count_elements++;
         return count_elements;
      }
      
      int& front() { return *this->f.begin(); }
      int& back() {
         return *this->back_iterator();
      }
      std::forward_list<int>::iterator back_iterator() {
         auto predecessor = this->f.begin();
         for(auto present = this->f.begin(); present != this->f.end(); present++)
            predecessor = present;
         return predecessor;
      }
      
      int& at(int i) { return *this->index(i); }
      std::forward_list<int>::iterator index(int i) {
         auto j = this->f.begin();
         for(int k = 0; k < i; k++) j++;
         return j;
      }

      void push(const int& pushed_item) { this->push_front(pushed_item); }
      void push_front(const int& pushed_item) { this->f.push_front(pushed_item); }
      void push_back(const int& pushed_item) {
         if(this->empty()) this->push_front(pushed_item);
         else this->f.insert_after(this->back_iterator(), pushed_item);
      }

      /* for consistency with own SinglyLinkedList */
      void pop() { this->pop_front(); }
      
      void pop_front() { this->f.pop_front(); }
      void pop_back() {
         auto second_before_present = this->f.begin();
         auto predecessor = this->f.begin();
         auto present = predecessor;
         present++;
         while(present != this->f.end())
         {
            second_before_present = predecessor;
            predecessor = present;
            present++;
         }
         this->f.erase_after(second_before_present);
      }
      void clear() { this->f.clear(); }

      void insert_at(int i, const int& inserted_item) {
         if(i == 0) this->f.push_front(inserted_item);
         else this->f.insert_after(this->index(i-1), inserted_item);
      }
      void erase_at(int i) {
         if(i == 0) this->f.pop_front();
         else this->f.erase_after(this->index(i-1));
      }

   private:
      std::forward_list<int> f = std::forward_list<int>();
   };
}

#endif
