#ifndef STL_VECTOR_WRAPPER_H
#define STL_VECTOR_WRAPPER_H

#include <vector>

namespace seq
{
   class STLVectorWrapper: public Sequence
   {
   public:
      bool empty() const { return this->v.empty(); }
      size_t size() const { return this->v.size(); }
      
      int& front() { return this->v.front(); }
      int& back() { return this->v.back(); }
      
      int& at(int i) { return this->v.at(i); }

      /* for consistency with own DynamicArray */
      void push(const int& pushed_item) { this->push_back(pushed_item); }
      
      void push_front(const int& pushed_item) { this->insert_at(0, pushed_item); }
      void push_back(const int& pushed_item) { this->v.push_back(pushed_item); }

      /* for consistency with own DynamicArray */
      void pop() { this->pop_back(); }
      
      void pop_front() { this->erase_at(0); }
      void pop_back() { this->v.pop_back(); }
      void clear() { this->v.clear(); }

      void insert_at(int i, const int& inserted_item) {
         this->v.insert(this->v.begin() + i, inserted_item);
      }
      void erase_at(int i) { this->v.erase(this->v.begin() + i); }

   private:
      std::vector<int> v = std::vector<int>();
   };
}

#endif
